/*
	Ogg Vorbis CODEC for MSACM
	Copyright (C)2001 H.Mutsuki
*/

#include "hmc/util.h"
#include "hmc/mmutil.h"
#include "codec.h"
#include "encode.h"
#include "resource.h"

#pragma warning(push,1)
#include "vorbis/codec.h"
#include "vorbis/vorbisenc.h"
#pragma warning(pop)

#ifndef _DEBUG
#ifdef RELEASEDEBUG
#undef TRACE
#define TRACE HMC::Trace
#endif
#endif

/*****************************************************************
	Ogg Vorbis for MSACM
*****************************************************************/
/*****************************************************************
	Ogg Vorbis GR[h
*****************************************************************/
void COggVorbisEncode::init()
{
	// ̌{I(悢q͐^Ȃł)
	size_t n = offsetof(TYPE,pDriver) + sizeof(pDriver);
	memset((LPBYTE)this+n,0,sizeof(*this)-n);
}
void COggVorbisEncode::destroy()
{
	ogg_stream_clear(&os_pad);
	ogg_stream_clear(&os);
	vorbis_block_clear(&vb);
	vorbis_dsp_clear(&vd);
	vorbis_comment_clear(&vc);
	vorbis_info_clear(&vi);
	init();
}
LRESULT COggVorbisEncode::Create(CONST WAVEFORMATEX *pwfxSrc, CONST WAVEFORMATEX *pwfxDst)
{
	LRESULT lRes = MMSYSERR_NOERROR;
	destroy();
	do {
		if(pwfxDst->wFormatTag==WAVE_FORMAT_VORBIS1P ||
		   pwfxDst->wFormatTag==WAVE_FORMAT_VORBIS2P ||
		   pwfxDst->wFormatTag==WAVE_FORMAT_VORBIS3P) {
			dwFlags |= ENCODE_FLAGS_PSEUDOCBR;
		}
		if(pwfxSrc->wFormatTag==WAVE_FORMAT_VORBIS1 ||
		   pwfxSrc->wFormatTag==WAVE_FORMAT_VORBIS1P) {
			dwFlags |= ENCODE_FLAGS_MODE1;
		}
		if(pwfxSrc->wFormatTag==WAVE_FORMAT_VORBIS2 ||
		   pwfxSrc->wFormatTag==WAVE_FORMAT_VORBIS2P) {
			dwFlags |= ENCODE_FLAGS_MODE2;
		}
		if(pwfxSrc->wFormatTag==WAVE_FORMAT_VORBIS3 ||
		   pwfxSrc->wFormatTag==WAVE_FORMAT_VORBIS3P) {
			dwFlags |= ENCODE_FLAGS_MODE3;
		}
		// GR[h[h
		vorbis_info_init(&vi);
		WORD  c = pwfxDst->nChannels;
		DWORD s = pwfxDst->nSamplesPerSec;
		DWORD i = oggFormatToIndex(pwfxDst);
		DWORD r = aOggFormatIndexToDetail[i].nAvgBytesPerSec * 8;
		float q = aOggFormatIndexToDetail[i].flQuality;
		if(dwFlags & ENCODE_FLAGS_PSEUDOCBR) {
#if 0	// 肪̂œABR͎gpȂƂɁcc
			if(s>42000 && s<46000) {	// libVorbis I 20011231̏ꍇ44KHzȊOł̃rbg[gǗ̓T|[gȂ̂
				if(vorbis_encode_init(&vi,c,s,r,r,r)!=0) {
					lRes = ACMERR_NOTPOSSIBLE;
					break;
				}
				dwFlags |= ENCODE_FLAGS_RATEMANAGED;
			} else {
				if(vorbis_encode_init_vbr(&vi,c,s,q)!=0) {
					lRes = ACMERR_NOTPOSSIBLE;
					break;
				}
			}
#else
			if(vorbis_encode_init_vbr(&vi,c,s,q)!=0) {
				lRes = ACMERR_NOTPOSSIBLE;
				break;
			}
#endif
		} else {
			if(vorbis_encode_init_vbr(&vi,c,s,q)!=0) {
				lRes = ACMERR_NOTPOSSIBLE;
				break;
			}
		}
		// Rg
		vorbis_comment_init(&vc);
		vorbis_comment_add_tag(&vc,"ENCODER","vorbis.acm");
		// GR[_ԃf[^AXg[W
		vorbis_analysis_init(&vd,&vi);
		vorbis_block_init(&vd,&vb);
		// VAio[
		ogg_stream_init(&os,STREAMSERIALNO_AUDIO);			// I[fBI
		ogg_stream_init(&os_pad,STREAMSERIALNO_PADDING);	// pfBO
		// wb_쐬(K{̂R)
		ogg_packet header_main;			// Vorbiswb_
		ogg_packet header_comments;		// Rgwb_
		ogg_packet header_codebooks;	// R[hubNwb_
		// wb_pPbg쐬
		vorbis_analysis_headerout(&vd,&vc,&header_main,&header_comments,&header_codebooks);
		// wb_pPbgo
		ogg_stream_packetin(&os,&header_main);
		ogg_stream_packetin(&os,&header_comments);
		ogg_stream_packetin(&os,&header_codebooks);
	} while(0);
	if(lRes!=MMSYSERR_NOERROR) {
		destroy();
	}
	return lRes;
}
/*****************************************************************
	ϊR[obN
*****************************************************************/
LRESULT COggVorbisEncode::ConvertCallback(LPACMDRVSTREAMINSTANCE padsi, LPACMDRVSTREAMHEADER padsh)
{
	LRESULT lRes = MMSYSERR_NOERROR;

	// wb_o
	if(!(dwFlags & ENCODE_FLAGS_HEADERPREPARED)) {
		// mode2,mode3̏ꍇ̓wb_o͂̂Ă
		if(dwFlags & (ENCODE_FLAGS_MODE2|ENCODE_FLAGS_MODE3)) {
			while(1) {
				if(ogg_stream_flush(&os,&og)==0) {
					break;
				}
			}
			dwFlags |= ENCODE_FLAGS_HEADERPREPARED;
		} else {
			while(1) {
				if(0<cbDstLengthUsed) {
					TRACE("Continue Writing: %u/%u\n",cbDstLengthUsed,og.header_len+og.body_len);
					goto continue_write_header;
				}
				if(cbDstLengthUsed==0) {
					if(ogg_stream_flush(&os,&og)==0) {
						dwFlags |= ENCODE_FLAGS_HEADERPREPARED;
						break;
					}
				}
			continue_write_header:
				DWORD cbWrote = WritePage(padsi,padsh);
				if(cbWrote<(DWORD)og.header_len+og.body_len) {
					goto carry_header;	// wb_oz
				}
			}
			cbDstPrepared = 0;	// wb_TCYPADZoɊ܂߂Ȃ
		}
	}

	// ǂݍ݁ÁAo[v
	while(!(dwFlags&ENCODE_FLAGS_EOS)) {
		DWORD cbSamples = 0;
		if(0<cbDstLengthUsed) {
			TRACE("Continue Writing: %u/%u\n",cbDstLengthUsed,og.header_len+og.body_len);
			goto continue_write;
		}
		if(dwFlags & ENCODE_FLAGS_EOF) {
			dwFlags |= ENCODE_FLAGS_EOS;
		}
		if(0<padsh->cbSrcLength) {
			cbSamples = ReadData(padsi,padsh);
			if(0<cbSrcLengthUsed) {
				goto carry_read;	// f[^ǂݍݎz
			}
			if(cbSamples==0 && padsh->cbSrcLength==padsh->cbSrcLengthUsed) {
				if(padsh->fdwConvert & ACM_STREAMCONVERTF_END) {
					dwFlags |= ENCODE_FLAGS_EOF;
				} else {
					goto carry_read;	// f[^ǂݍݎz(̂Q)
				}
			}
		}
		vorbis_analysis_wrote(&vd,cbSamples);
		while(1) {
			if(vorbis_analysis_blockout(&vd,&vb)!=1) {
				if(padsh->cbSrcLength==0) {
					dwFlags |= ENCODE_FLAGS_EOS;
				}
				break;
			}
			vorbis_analysis(&vb,NULL);
			vorbis_bitrate_addblock(&vb);
			while(vorbis_bitrate_flushpacket(&vd,&op)) {
				ogg_stream_packetin(&os,&op);
				while(!(dwFlags&ENCODE_FLAGS_EOS)) {
					if(ogg_stream_pageout(&os,&og)==0) {
						break;
					}
				continue_write:
					DWORD cbWrote = WritePage(padsi,padsh);
					if(cbWrote<(DWORD)og.header_len+og.body_len) {
						goto carry_write;	// y[Woz
					}
					if(ogg_page_eos(&og)) {
						dwFlags |= ENCODE_FLAGS_EOS;
						break;
					}
				}
			}
		}
	}
 carry_read:
	// o͂OoCg̏ꍇXg[̏IƂ݂Ȃꍇ
	// Xg[Ih~̂ߋIɃI[fBI܂̓pfBOy[Wo
	// ͂Bitrate邪fR[hʂɉe͂Ȃ
	if(0<padsh->cbSrcLengthUsed && padsh->cbDstLengthUsed==0) {
	insert_padding:
		if(ogg_stream_flush(&os,&og)!=0) {
			DWORD cbWrote = WritePage(padsi,padsh);
			if(cbWrote<(DWORD)og.header_len+og.body_len) {
				goto carry_write;		// y[Woz
			}
		} else {
			op_pad.packet = NULL;
			op_pad.bytes  = 0;
			ogg_stream_packetin(&os_pad,&op_pad);
			if(ogg_stream_flush(&os_pad,&og)!=0) {
				if((DWORD)og.header_len+og.body_len<=padsh->cbDstLength-padsh->cbDstLengthUsed) {
					cbPadPrepared += og.header_len+og.body_len;
					DWORD cbWrote = WritePage(padsi,padsh);
					ASSERT((DWORD)og.header_len+og.body_len<=cbWrote);
				} else {
					goto carry_write;	// pfBO݌EɓBꍇ͔
				}
			}
		}
	}
	// [CBRppfBO}
	if(dwFlags & ENCODE_FLAGS_PSEUDOCBR) {
		DWORD cbSrcPerSec = HMC::GetAvgBytesPerSec(padsi->pwfxSrc);
		DWORD cbDstPerSec = HMC::GetAvgBytesPerSec(padsi->pwfxDst);
		DWORD dwEstimate = (DWORD)((QWORD)cbSrcPrepared * cbDstPerSec / cbSrcPerSec);
		DWORD cbMergin = (dwFlags & ENCODE_FLAGS_EOS) ? 0 : min(cbDstPerSec/16, padsh->cbDstLength/16);
		if(cbMergin<cbDstPrepared-cbPadPrepared && cbDstPrepared+cbMergin<dwEstimate) {
			goto insert_padding;	// 藬pcc
		}
	}
 carry_write:
	// Iɓ͂S
	// Ih~pc]Ƀ
	// ꍇɂẮAώZIɃʂ邱Ƃcc
	while(padsh->cbSrcLengthUsed < padsh->cbSrcLength) {
		DWORD cbSamples = 0;
		if(0<padsh->cbSrcLength) {
			cbSamples = ReadData(padsi,padsh);
			if(0<cbSrcLengthUsed) {
				//goto carry_write;	// f[^ǂݍݎz
				goto carry_write_carry;
			}
			if(cbSamples==0 && padsh->cbSrcLength==padsh->cbSrcLengthUsed) {
				//goto carry_write;	// f[^ǂݍݎz(̂Q)
				goto carry_write_carry;
			}
		}
		vorbis_analysis_wrote(&vd,cbSamples);
	}
 carry_write_carry:
 carry_header:
	// eI[o[AA_[`FbN
	ASSERT(padsh->cbSrcLengthUsed <= padsh->cbSrcLength);
	ASSERT(padsh->cbDstLengthUsed <= padsh->cbDstLength);
	// ͂SɏȂƎcf[^hbvꍇ
	if(padsh->cbSrcLengthUsed < padsh->cbSrcLength) {
		TRACE("Input data may be dropped %u[bytes] (frame %ld)\n",padsh->cbSrcLength-padsh->cbSrcLengthUsed,(long)(vd.sequence));
	}
	// o͂OoCg̏ꍇXg[̏IƂ݂Ȃꍇ
	if(0<padsh->cbSrcLength && 0==padsh->cbDstLengthUsed) {
		TRACE("Output data may be dropped (frame %ld)\n",(long)(vd.sequence));
	}

	// rbg[gpf[^XV
#if defined(_DEBUG) || defined(RELEASEDEBUG)
	float flLastBitrate = flCurBitrate;
	flEstBitrate = (float)((LONG64)cbDstPrepared*HMC::GetAvgBytesPerSec(padsi->pwfxSrc)*8) / max(1,cbSrcPrepared) / 1000;
	flCurBitrate = (float)((LONG64)(cbDstPrepared-cbDstLastPrepared)*HMC::GetAvgBytesPerSec(padsi->pwfxSrc)*8) / max(1,cbSrcPrepared-cbSrcLastPrepared) / 1000;
	flMaxBitrate = max(flMaxBitrate,flCurBitrate);
	cbSrcLastPrepared = cbSrcPrepared;
	cbDstLastPrepared = cbDstPrepared;
	cbPadLastPrepared = cbPadPrepared;
#endif
	TRACE("Converting: %u/%u/%u, Est:%.2f, Cur:%.2f, Max:%.2f\n",cbSrcPrepared,cbDstPrepared,cbPadPrepared,flEstBitrate,flCurBitrate,flMaxBitrate);
	return lRes;
}
/*****************************************************************
	PCMf[^ǂݍ
*****************************************************************/
DWORD COggVorbisEncode::ReadData(LPACMDRVSTREAMINSTANCE padsi, LPACMDRVSTREAMHEADER padsh)
{
	DWORD cbSamples = READSIZE;
	DWORD cbUnit = HMC::GetBytesPerSample(padsi->pwfxSrc);
	DWORD cbSrcLengthRemain = padsh->cbSrcLength - padsh->cbSrcLengthUsed;
	cbSamples = min(cbSamples,(cbSrcLengthUsed+cbSrcLengthRemain)/cbUnit);

	// RAWPCM(16Bits)͗pfloatobt@ɓWJ
	float **ppfDst = vorbis_analysis_buffer(&vd,READSIZE);	// ͗pfloatobt@
	DWORD cbOffset = cbSrcLengthUsed / cbUnit;				// ͗pfloatobt@̃ItZbg
	LPBYTE pbSrc = padsh->pbSrc + padsh->cbSrcLengthUsed;	// acmPCM̓obt@
	DWORD cbRead = min(cbSamples-cbOffset,cbSrcLengthRemain/cbUnit);	// ǂݍݕ̃Tv
	if(vi.channels==1) {
		for(DWORD n=0; n<cbRead; n++) {
			ppfDst[0][cbOffset+n] = (SHORT)(((CHAR*)pbSrc)[n*2+1]<<8|pbSrc[n*2+0]) / 32768.0f;
			ASSERT(-1.0f<=ppfDst[0][cbOffset+n] && ppfDst[0][cbOffset+n]<=1.0f);
		}
	} else if(vi.channels==2) {
		for(DWORD n=0; n<cbRead; n++) {
			ppfDst[0][cbOffset+n] = (SHORT)(((CHAR*)pbSrc)[n*4+1]<<8|pbSrc[n*4+0]) / 32768.0f;
			ppfDst[1][cbOffset+n] = (SHORT)(((CHAR*)pbSrc)[n*4+3]<<8|pbSrc[n*4+2]) / 32768.0f;
			ASSERT(-1.0f<=ppfDst[0][cbOffset+n] && ppfDst[0][cbOffset+n]<=1.0f);
			ASSERT(-1.0f<=ppfDst[1][cbOffset+n] && ppfDst[1][cbOffset+n]<=1.0f);
		}
	}

	// eJ[\XV
	padsh->cbSrcLengthUsed += cbRead * cbUnit;	// acmPCM̓ItZbg
	cbSrcLengthUsed        += cbRead * cbUnit;	// ͗pfloatItZbg
	cbSrcPrepared          += cbRead * cbUnit;	// pfBOZopS

	if(cbRead==0 && !(padsh->fdwConvert & ACM_STREAMCONVERTF_END)) {
		return 0;
	} else if(cbOffset+cbRead<cbSamples) {
		TRACE("Carried(R): %u/%u\n",cbRead,cbSamples);
		return cbRead;			// ǂݍ݃TvԂ()
	} else {
		ASSERT(cbOffset+cbRead==cbSamples);
		cbSrcLengthUsed = 0;	// σoCgNA
		//vorbis_analysis_wrote(&vd,cbSamples);
		return cbSamples;		// ǂݍ݃TvԂ(S)
	}
}
/*****************************************************************
	y[Wo
*****************************************************************/
DWORD COggVorbisEncode::WritePage(LPACMDRVSTREAMINSTANCE padsi, LPACMDRVSTREAMHEADER padsh)
{
	DWORD len1 = og.header_len;	// wb_
	DWORD len2 = og.body_len;	// {fB

	// wb_o
	if(cbDstLengthUsed < len1) {
		DWORD len = padsh->cbDstLength - padsh->cbDstLengthUsed;
		len = min(len, len1-cbDstLengthUsed);
		LPBYTE pd = padsh->pbDst + padsh->cbDstLengthUsed;
		LPBYTE ps = (LPBYTE)og.header + cbDstLengthUsed;
		memcpy(pd,ps,len);
		padsh->cbDstLengthUsed += len;
		cbDstLengthUsed += len;
		cbDstPrepared += len;
	}
	// {fBo
	if(cbDstLengthUsed < len1+len2) {
		DWORD len = padsh->cbDstLength - padsh->cbDstLengthUsed;
		len = min(len, len2-(cbDstLengthUsed-len1));
		LPBYTE pd = padsh->pbDst + padsh->cbDstLengthUsed;
		LPBYTE ps = (LPBYTE)og.body + (cbDstLengthUsed-len1);
		memcpy(pd,ps,len);
		padsh->cbDstLengthUsed += len;
		cbDstLengthUsed += len;
		cbDstPrepared += len;
	}

	if(cbDstLengthUsed < len1+len2) {
		TRACE("Carried(W): %u/%u\n",len1+len2-cbDstLengthUsed,len1+len2);
		return cbDstLengthUsed;	// ݃oCgԂ()
	} else {
		cbDstLengthUsed = 0;	// ς݃oCgNA
		return len1+len2;		// ݃oCgԂ(S)
	}
}
/*****************************************************************
*****************************************************************/

/* Local Variables:						*/
/* tab-width:4							*/
/* c-basic-offset:4						*/
/* c-indent-level:4						*/
/* c-label-offset:-4					*/
/* c-continued-statement-offset:4		*/
/* file-coding-system:*sjis*dos			*/
/* buffer-file-coding-system:sjis-dos	*/
/* End:									*/
